@@ -1,5 +1,7 @@
 """Scalene: a high-performance, high-precision CPU *and* memory profiler for Python.
 
+    https://github.com/emeryberger/scalene
+
     Scalene uses interrupt-driven sampling for CPU profiling. For memory
     profiling, it uses a similar mechanism but with interrupts generated
     by a "sampling memory allocator" that produces signals everytime the
@@ -20,6 +22,7 @@ import builtins
 import contextlib
 import dis
 import os
+import platform
 import signal
 import subprocess
 import sys
@@ -30,6 +33,7 @@ from collections import defaultdict
 from contextlib import contextmanager
 from functools import lru_cache
 from rich.console import Console
+from rich.markdown import Markdown
 from rich.syntax import Syntax
 from rich.table import Table
 from rich.text import Text
@@ -54,8 +58,92 @@ from typing import (
     cast,
 )
 
-from . import adaptive
-from . import sparkline
+# from . import adaptive
+# from . import sparkline
+
+
+class adaptive:
+    """Implements sampling to achieve the effect of a uniform random sample."""
+
+    sample_array: List[float] = []
+    current_index = 0
+    total_samples = 0
+    max_samples = 0
+    count_average = 0
+    sum_average = 0
+    max_average = 1
+
+    def __init__(self, size: int):
+        self.max_samples = size
+        # must be a power of two
+        self.sample_array = [0] * size
+
+    def add(self, value: float) -> None:
+        if self.current_index >= self.max_samples:
+            # Decimate
+            # print("DECIMATION " + str(self.sample_array))
+            new_array = [0.0] * self.max_samples
+            for i in range(0, self.max_samples // 3):
+                arr = [self.sample_array[i * 3 + j] for j in range(0, 3)]
+                arr.sort()
+                new_array[i] = arr[1]  # Median
+            self.current_index = self.max_samples // 3
+            self.sample_array = new_array
+            # print("POST DECIMATION = " + str(self.sample_array))
+            # Update average length
+            self.max_average *= 3
+        self.sample_array[self.current_index] = value
+        self.current_index += 1  # count_average += 1
+
+    def get(self) -> List[float]:
+        return self.sample_array
+
+    def len(self) -> int:
+        return self.current_index
+
+
+# Sparkline stuff
+
+# Check if we are in Windows Subsystem for Linux and *not* using
+# the highly recommended Windows Terminal
+# (https://aka.ms/windowsterminal)
+if "WSL_DISTRO_NAME" in os.environ and "WT_PROFILE_ID" not in os.environ:
+    # We are running in the Windows Subsystem for Linux Display, a
+    # crappy version of the sparkline because the Windows console
+    # *still* only properly displays IBM Code page 437 by default.
+    # ▄▄■■■■▀▀
+    bar = chr(0x2584) * 2 + chr(0x25A0) * 3 + chr(0x2580) * 3
+else:
+    # Reasonable system. Use Unicode characters.
+    # Unicode: 9601, 9602, 9603, 9604, 9605, 9606, 9607, 9608
+    # ▁▂▃▄▅▆▇█
+    bar = "".join([chr(i) for i in range(9601, 9609)])
+
+barcount = len(bar)
+
+# From https://rosettacode.org/wiki/Sparkline_in_unicode#Python
+def sparkline(
+    numbers: List[float], fixed_min: float = -1, fixed_max: float = -1
+) -> Tuple[float, float, str]:
+    if fixed_min == -1:
+        mn = float(min(numbers))
+    else:
+        mn = fixed_min
+    if fixed_max == -1:
+        mx = float(max(numbers))
+    else:
+        mx = fixed_max
+    # print(numbers)
+    # mn, mx = min(numbers), max(numbers)
+    extent = mx - mn
+    if extent == 0:
+        extent = 1
+    # print("mn, mx = " + str(mn) + ", " + str(mx) + " extent = " + str(extent))
+    sparkstr = "".join(
+        bar[min([barcount - 1, int((n - mn) / extent * barcount)])] for n in numbers
+    )
+    return mn, mx, sparkstr
+
 
 # Logic to ignore @profile decorators.
 try:
@@ -143,7 +231,15 @@ def parse_args() -> Tuple[argparse.Namespace, List[str]]:
 arguments, left = parse_args()
 
 # Load shared objects unless the user specifies "--cpu-only" at the command-line.
-if not arguments.cpuonly:
+# (x86-64 only for now.)
+
+if not arguments.cpuonly and platform.machine() != "x86_64":
+    arguments.cpuonly = True
+    print(
+        "scalene warning: currently only x86-64 platforms are supported for memory and copy profiling."
+    )
+
+if not arguments.cpuonly and platform.machine() == "x86_64":
     # Load the shared object on Linux.
     if sys.platform == "linux":
         if ("LD_PRELOAD" not in os.environ) and ("PYTHONMALLOC" not in os.environ):
@@ -342,11 +438,11 @@ class Scalene:
     __last_signal_time: float = 0
 
     # memory footprint samples (time, footprint), using 'adaptive' sampling.
-    __memory_footprint_samples = adaptive.adaptive(27)
+    __memory_footprint_samples = adaptive(27)
 
     # same, but per line
-    __per_line_footprint_samples: Dict[str, Dict[int, adaptive.adaptive]] = defaultdict(
-        lambda: defaultdict(lambda: adaptive.adaptive(9))
+    __per_line_footprint_samples: Dict[str, Dict[int, adaptive]] = defaultdict(
+        lambda: defaultdict(lambda: adaptive(9))
     )
 
     # original working directory
@@ -909,12 +1005,7 @@ process."""
         if "site-packages" in filename or "/usr/lib/python" in filename:
             # Don't profile Python internals.
             return False
-        if (
-            "scalene.py" in filename
-            or "adaptive.py" in filename
-            or "sparkline.py" in filename
-            or "scalene/__main__.py" in filename
-        ):
+        if "scalene.py" in filename or "scalene/__main__.py" in filename:
             # Don't profile the profiler.
             return False
         filename = os.path.abspath(filename)
@@ -956,18 +1047,12 @@ process."""
             return 0, 0, ""
         # Prevent negative memory output due to sampling error.
         samples = [i if i > 0 else 0 for i in arr]
-        minval, maxval, sp_line = sparkline.sparkline(
-            samples[0:iterations], minimum, maximum
-        )
+        minval, maxval, sp_line = sparkline(samples[0:iterations], minimum, maximum)
         return minval, maxval, sp_line
 
     @staticmethod
     def output_profile_line(
-        fname: Filename,
-        line_no: LineNumber,
-        line: str,
-        console: Console,
-        tbl: Table,
+        fname: Filename, line_no: LineNumber, line: str, console: Console, tbl: Table,
     ) -> None:
         """Print exactly one line of the profile."""
         current_max = Scalene.__max_footprint
@@ -978,7 +1063,9 @@ process."""
         line = line.rstrip()
         # Generate syntax highlighted version.
         if Scalene.__html:
-            syntax_highlighted = Syntax(line, "python", theme="default", line_numbers=False)
+            syntax_highlighted = Syntax(
+                line, "python", theme="default", line_numbers=False
+            )
         else:
             syntax_highlighted = Syntax(line, "python", theme="vim", line_numbers=False)
         # Prepare output values.
@@ -1067,9 +1154,9 @@ process."""
                 )
 
             # Red highlight
-            ncpps : Any = ""
-            ncpcs : Any = ""
-            nufs : Any = ""
+            ncpps: Any = ""
+            ncpcs: Any = ""
+            nufs: Any = ""
             if (
                 n_usage_fraction >= Scalene.__highlight_percentage
                 or (n_cpu_percent_c + n_cpu_percent_python)
@@ -1144,7 +1231,7 @@ process."""
         ) > 0
         title = Text()
         # title.append("scalene\n", style="bold")
-        mem_usage_line = ""
+        mem_usage_line: Union[Text, str] = ""
         if did_sample_memory:
             samples = Scalene.__memory_footprint_samples
             if len(samples.get()) > 0:
@@ -1152,8 +1239,11 @@ process."""
                 _, _, spark_str = Scalene.generate_sparkline(
                     samples.get()[0 : samples.len()], 0, current_max
                 )
-                mem_usage_line = Text.assemble("Memory usage: ", ((spark_str, "blue")),
-                                               (" (max: %6.2fMB)\n" % current_max))
+                mem_usage_line = Text.assemble(
+                    "Memory usage: ",
+                    ((spark_str, "blue")),
+                    (" (max: %6.2fMB)\n" % current_max),
+                )
                 title.append(mem_usage_line)
 
         null = open("/dev/null", "w")
@@ -1166,9 +1256,7 @@ process."""
             )
 
             try:
-                percent_cpu_time = (
-                    100 * this_cpu_samples / Scalene.__total_cpu_samples
-                )
+                percent_cpu_time = 100 * this_cpu_samples / Scalene.__total_cpu_samples
             except ZeroDivisionError:
                 percent_cpu_time = 0
 
@@ -1178,12 +1266,16 @@ process."""
                 continue
 
             # Print header.
-            new_title = mem_usage_line + ("%s: %% of CPU time = %6.2f%% out of %6.2fs."
-                                 % (fname, percent_cpu_time, Scalene.__elapsed_time))
+            new_title = mem_usage_line + (
+                "%s: %% of CPU time = %6.2f%% out of %6.2fs."
+                % (fname, percent_cpu_time, Scalene.__elapsed_time)
+            )
             # Only display total memory usage once.
             mem_usage_line = ""
 
-            tbl = Table(box=box.MINIMAL_HEAVY_HEAD, title=new_title, collapse_padding=True)
+            tbl = Table(
+                box=box.MINIMAL_HEAVY_HEAD, title=new_title, collapse_padding=True
+            )
 
             tbl.add_column("Line", justify="right", no_wrap=True)
             tbl.add_column("CPU %\nPython", no_wrap=True)
@@ -1201,11 +1293,15 @@ process."""
                         fname, LineNumber(line_no), line, console, tbl
                     )
             console.print(tbl)
-            
+
         if Scalene.__html:
             # Write HTML file.
             if not Scalene.__output_file:
                 Scalene.__output_file = "/dev/stdout"
+            md = Markdown(
+                "generated by the [scalene](https://github.com/emeryberger/scalene) profiler"
+            )
+            console.print(md)
             console.save_html(Scalene.__output_file, clear=False)
         else:
             if not Scalene.__output_file:
